/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package de.hillenbrand.dupfinder;

import de.hillenbrand.system.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.GridBagLayout;
import java.io.*;
import java.io.BufferedInputStream;
import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.net.*;
import java.text.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.ImageIcon;
import jreqtools.*;

class JDupFinderResultFrame extends JFrame {
	// App Prefs
	private ApplicationPreferences prefs = null;
	
	// Variables
	private Vector	duplicateFiles	= null;
	
	// Widgets
	final JList		list1			= new JList();
	final JList		list2			= new JList();
	final JButton 	showButton 	= new JButton("Show");
	final JButton 	deleteButton 	= new JButton("Delete");
	
	public JDupFinderResultFrame(Vector duplicateFiles)  {
		super("JDupFinder: Results");
		
		// init
		this.duplicateFiles = duplicateFiles;
		
		// Properties
		this.setDefaultCloseOperation(DISPOSE_ON_CLOSE);
		list1.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		list2.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
		
		// Listeners
		list1.addListSelectionListener(new ListSelectionListener() { public void valueChanged(ListSelectionEvent e) { updateList2(e); }});
		list2.addMouseListener(new MouseAdapter() { public void mouseClicked(MouseEvent e) { if (e.getClickCount() == 2) showFiles(); }});
		showButton.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { showFiles(); }});
		deleteButton.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { deleteFiles(); }});
		
		// create and set panel
		final JPanel panel = createContentPanel();
		panel.setBorder(UIPreferences.getFrameBorder());
		setContentPane(panel);
		
		// set data
		Vector v = new Vector();
		for (int i=0; i<duplicateFiles.size(); ++i) v.addElement(((Vector)duplicateFiles.elementAt(i)).elementAt(0));
		list1.setListData(v);
		list1.setSelectedIndex(0);
		
		// calculate frame size
		pack();
		JDupFinder.addPrefs("ResultFrame", this);
	}

	JPanel createContentPanel()  {
		final JPanel panel = new JPanel(new GridBagLayout());
		final GridBagConstraints c = UIPreferences.getGridBagConstraints();
		
		// Lists
		c.fill = GridBagConstraints.BOTH;
		c.gridx = 0; c.gridy=0; c.gridwidth=1; c.gridheight=1; c.weightx=1; c.weighty=1;
		panel.add(new JScrollPane(list1, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED), c);
		c.gridx = 1; c.gridy=0; c.gridwidth=3; c.gridheight=1; c.weightx=1; c.weighty=1;
		panel.add(new JScrollPane(list2, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED), c);
		
		// Button
		c.fill = GridBagConstraints.HORIZONTAL;
		c.gridx = 1; c.gridy=1; c.gridwidth=1; c.gridheight=1; c.weightx=1; c.weighty=0;
		panel.add(new JPanel(), c);
		c.gridx = 2; c.gridy=1; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		panel.add(deleteButton, c);
		c.gridx = 3; c.gridy=1; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		panel.add(showButton, c);
		
		return panel;
	}

	void deleteFiles()  {
		// get selected items
		Object [] selected = list2.getSelectedValues();
		if (selected == null) return;
		
		// delete each files
		for (int i=0; i<selected.length; ++i)
			{
			JDupFinderFile file = (JDupFinderFile) selected[i];
			File f = new File(file.filename);
			if (f.delete())
				{
				// update the data model
				int index = list1.getSelectedIndex(); // is always >= 0 !!
				// get and set the data for the second list
				Vector v = (Vector) duplicateFiles.elementAt(index);
				v.removeElement(file);
				list2.setListData(v);
				}
			else JReqTools.showError("Could not delete '"+file.filename+"'");
			}
		
		// if there is only one item left select next possible duplicate's entry
		if (list2.getModel().getSize() < 2)
			{
			int selectedIndex = list1.getSelectedIndex();
			if (selectedIndex < list1.getModel().getSize() - 1) 
				{
				Object toSelect = list1.getModel().getElementAt(selectedIndex+1);
				list1.setSelectedValue(toSelect, true);
				}
			}
	}

	void showFiles()  {
		// get selected items
		Object [] selected = list2.getSelectedValues();
		if (selected == null) return;
		
		// show each files
		for (int i=0; i<selected.length; ++i)
			{
			JDupFinderFile file = (JDupFinderFile) selected[i];
			JDupFinderShowFrame  f = new JDupFinderShowFrame(file.filename);
			f.setVisible(true);
			}
	}

	void updateList2(ListSelectionEvent e)  {
		// check if there are more events to come
		if (e != null && e.getValueIsAdjusting()) return;
		
		// index of selected item
		int index = list1.getSelectedIndex();
		if (index < 0) 
			{
			list2.setListData(new Vector());
			return;
			}
		
		// get and set the data for the second list
		Vector v = (Vector) duplicateFiles.elementAt(index);
		list2.setListData(v);
		
		// select all but first
		int [] select = new int[v.size()-1];
		for (int i=0; i<select.length; ++i) select[i] = i+1;
		list2.setSelectedIndices(select);
	}

}
