/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package de.hillenbrand.system;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.text.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import jreqtools.*;

public final class UIPreferences {
	private static final boolean DEBUG = false;

	// App Prefs
	private static final ApplicationPreferences prefs = new ApplicationPreferences("JavaUI");
	
	public static final Font getAcceleratorFont()  {
		Font result = prefs.getFont("AcceleratorFont");
		return result != null ? result : new Font("Verdana", Font.PLAIN, 10);
	}

	public static final Font getFont()  {
		Font result = prefs.getFont("Font");
		return result != null ? result : new Font("Verdana", Font.PLAIN, 12);
	}

	public static final Border getFrameBorder()  {
		return BorderFactory.createEmptyBorder(10,10,10,10);
	}

	public static final GridBagConstraints getGridBagConstraints()  {
		final GridBagConstraints result = new GridBagConstraints();
		
		result.insets = new Insets(4,4,4,4);
		result.fill = GridBagConstraints.BOTH;
		
		return result;
	}

	public static final String getLookAndFeel()  {
		String result = prefs.getText("LookAndFeel");
		return result != null ? result : UIManager.getSystemLookAndFeelClassName();
	}

	public static final void init()  {
		// Look & Feel
		UIPreferences.setLookAndFeel(getLookAndFeel());
		
		// Fonts
		UIPreferences.setFont(getFont());
		UIPreferences.setAcceleratorFont(getAcceleratorFont());
	}

	static void save()  {
		prefs.save();
	}

	public static final void setAcceleratorFont(Font font)  {
		UIDefaults defaults = UIManager.getDefaults();
		
		defaults.put("CheckBoxMenuItem.acceleratorFont", font);
		defaults.put("Menu.acceleratorFont", font);
		defaults.put("MenuItem.acceleratorFont", font);
		defaults.put("RadioButtonMenuItem.acceleratorFont", font);
		
		prefs.setFont("AcceleratorFont", font);
	}

	public static final void setFont(Font font)  {
		UIDefaults defaults = UIManager.getDefaults();
		
		defaults.put("Button.font", font);
		defaults.put("CheckBox.font", font);
		defaults.put("CheckBoxMenuItem.font", font);
		defaults.put("ColorChooser.font", font);
		defaults.put("ComboBox.font", font);
		defaults.put("EditorPane.font", font);
		defaults.put("InternalFrame.titleFont", font);
		defaults.put("Label.font", font);
		defaults.put("List.font", font);
		defaults.put("Menu.font", font);
		defaults.put("MenuBar.font", font);
		defaults.put("MenuItem.font", font);
		defaults.put("OptionPane.font", font);
		defaults.put("Panel.font", font);
		defaults.put("PasswordField.font", font);
		defaults.put("PopupMenu.font", font);
		defaults.put("ProgressBar.font", font);
		defaults.put("RadioButton.font", font);
		defaults.put("RadioButtonMenuItem.font", font);
		defaults.put("ScrollPane.font", font);
		defaults.put("TabbedPane.font", font);
		defaults.put("Table.font", font);
		defaults.put("TableHeader.font", font);
		defaults.put("TextArea.font", font);
		defaults.put("TextField.font", font);
		defaults.put("TextPane.font", font);
		defaults.put("TitledBorder.font", font);
		defaults.put("ToggleButton.font", font);
		defaults.put("ToolBar.font", font);
		defaults.put("ToolTip.font", font);
		defaults.put("Tree.font", font);
		defaults.put("Viewport.font", font);
		
		prefs.setFont("Font", font);
	}

	public static final void setLookAndFeel(String lookAndFeel)  {
		try
			{
			UIManager.setLookAndFeel(lookAndFeel);
			}
		catch (Exception x) { JReqTools.showError(x.getMessage()); }
		
		prefs.setText("LookAndFeel", lookAndFeel);
	}

}
