/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package de.hillenbrand.system;

import java.awt.*;
import java.awt.event.*;
import java.awt.event.ActionListener;
import java.awt.GridBagLayout;
import java.io.*;
import java.text.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import jreqtools.*;
import java.text.MessageFormat;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

public class UIPreferencesFrame extends JFrame {
	private static final boolean DEBUG = false;

	// App Prefs
	private static final ApplicationPreferences prefs = new ApplicationPreferences("UIPreferencesFrame");
	
	// Variables
	Font font 				= UIPreferences.getFont();
	Font acceleratorFont 	= UIPreferences.getAcceleratorFont();
	String lookAndFeel 		= UIPreferences.getLookAndFeel();
	
	// Ressourcen
	final static ResourceBundle resources = ResourceBundle.getBundle("de.hillenbrand.system.resources.system");
	
	public class ContentPanel extends JPanel {
		JTabbedPane	tabbedPane	= new JTabbedPane();
		JButton 		saveButton 	= new JButton(UIPreferencesFrame.GRS("Button.Save"));
		JButton 		cancelButton 	= new JButton(UIPreferencesFrame.GRS("Button.Cancel"));
		
		public ContentPanel()  {
			super(new GridBagLayout());
			
			// Components
			initComponents();
			
			// Border
			this.setBorder(UIPreferences.getFrameBorder());
			
			// Content
			GridBagConstraints c = UIPreferences.getGridBagConstraints();
			
			c.fill = GridBagConstraints.BOTH;
			c.gridx = 0; c.gridy=0; c.gridwidth=3; c.gridheight=1; c.weightx=1; c.weighty=1;
			this.add(tabbedPane, c);
			
			c.fill = GridBagConstraints.HORIZONTAL;
			c.gridx = 0; c.gridy=1; c.gridwidth=1; c.gridheight=1; c.weightx=1; c.weighty=0;
			this.add(saveButton, c);
			
			c.fill = GridBagConstraints.HORIZONTAL;
			c.gridx = 1; c.gridy=1; c.gridwidth=1; c.gridheight=1; c.weightx=10; c.weighty=0;
			this.add(new JPanel(), c);
			
			c.fill = GridBagConstraints.HORIZONTAL;
			c.gridx = 2; c.gridy=1; c.gridwidth=1; c.gridheight=1; c.weightx=1; c.weighty=0;
			this.add(cancelButton, c);
		}

		void initComponents()  {
			tabbedPane.addTab(UIPreferencesFrame.GRS("TabbedPane.Fonts"), new FontPanel());
			tabbedPane.addTab(UIPreferencesFrame.GRS("TabbedPane.LookAndFeel"), new LookAndFeelPanel());
			
			// Listeners
			saveButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					UIPreferences.setFont(font);
					UIPreferences.setAcceleratorFont(acceleratorFont);
					UIPreferences.setLookAndFeel(lookAndFeel);
					UIPreferences.save();
					quit();
				}
			});
			cancelButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					quit();
				}
			});
		}

		void updateComponents()  {
		}

	}

	public class FontPanel extends JPanel {
		JButton fontButton 			= new JButton(font.getName() + "/" + font.getSize());
		JButton acceleratorFontButton 	= new JButton(acceleratorFont.getName() + "/" + acceleratorFont.getSize());
		
		public FontPanel()  {
			super(new GridBagLayout());
			
			// Components
			initComponents();
			
			// Border
			this.setBorder(UIPreferences.getFrameBorder());
			
			// Content
			GridBagConstraints c = UIPreferences.getGridBagConstraints();
			
			c.fill = GridBagConstraints.HORIZONTAL;
			c.gridx = 0; c.gridy=0; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
			this.add(new JLabel(UIPreferencesFrame.GRS("Label.StandardFont"), JLabel.LEFT), c);
			
			c.fill = GridBagConstraints.HORIZONTAL;
			c.gridx = 1; c.gridy=0; c.gridwidth=1; c.gridheight=1; c.weightx=1; c.weighty=0;
			this.add(fontButton, c);
			
			c.fill = GridBagConstraints.HORIZONTAL;
			c.gridx = 0; c.gridy=1; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
			this.add(new JLabel(UIPreferencesFrame.GRS("Label.AcceleratorFont"), JLabel.LEFT), c);
			
			c.fill = GridBagConstraints.HORIZONTAL;
			c.gridx = 1; c.gridy=1; c.gridwidth=1; c.gridheight=1; c.weightx=1; c.weighty=0;
			this.add(acceleratorFontButton, c);
		}

		void initComponents()  {
			// Listeners
			fontButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					FontRequester fr = new FontRequester(font);
					if (fr.request(UIPreferencesFrame.this) != FontRequester.OK) return;
					font = fr.getFont();
					updateComponents();
				}
			});
			
			acceleratorFontButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					FontRequester fr = new FontRequester(acceleratorFont);
					if (fr.request(UIPreferencesFrame.this) != FontRequester.OK) return;
					acceleratorFont = fr.getFont();
					updateComponents();
				}
			});
		}

		void updateComponents()  {
			fontButton.setText(font.getName() + "/" + font.getSize());
			acceleratorFontButton.setText(acceleratorFont.getName() + "/" + acceleratorFont.getSize());
		}

	}

	public class LookAndFeelPanel extends JPanel {
		JButton lookAndFeelButton = new JButton(lookAndFeel);
		
		public LookAndFeelPanel()  {
			super(new GridBagLayout());
			
			// Components
			initComponents();
			
			// Border
			this.setBorder(UIPreferences.getFrameBorder());
			
			// Content
			GridBagConstraints c = UIPreferences.getGridBagConstraints();
			
			c.fill = GridBagConstraints.HORIZONTAL;
			c.gridx = 0; c.gridy=0; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
			this.add(new JLabel(UIPreferencesFrame.GRS("Label.LookAndFeel"), JLabel.LEFT), c);
			
			c.fill = GridBagConstraints.HORIZONTAL;
			c.gridx = 1; c.gridy=0; c.gridwidth=1; c.gridheight=1; c.weightx=1; c.weighty=0;
			this.add(lookAndFeelButton, c);
		}

		void initComponents()  {
			// Listeners
			lookAndFeelButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					LookAndFeelRequester lfr = new LookAndFeelRequester();
					if (lfr.request(UIPreferencesFrame.this) != LookAndFeelRequester.OK) return;
					lookAndFeel = lfr.getLookAndFeel();
					updateComponents();
				}
			});
		}

		void updateComponents()  {
			lookAndFeelButton.setText(lookAndFeel);
		}

	}

	class MenuBar extends JMenuBar {
		JMenu menu_file = new JMenu(UIPreferencesFrame.GRS("Menu.Program"));
		
		JMenuItem menuItem_about = new JMenuItem(UIPreferencesFrame.GRS("MenuItem.About"));
		JMenuItem menuItem_quit = new JMenuItem(UIPreferencesFrame.GRS("MenuItem.Quit"));
		
		public MenuBar()  {
			// create menu: "File"
			menu_file.add(menuItem_about);
			menu_file.addSeparator();
			menu_file.add(menuItem_quit);
			this.add(menu_file);
			
			// Listeners
			menuItem_quit.addActionListener(new ActionListener() { 
				public void actionPerformed(ActionEvent e) { 
					quit();
				} 
			});
			menuItem_about.addActionListener(new ActionListener() { 
				public void actionPerformed(ActionEvent e) { 
					JReqTools.showMessage(UIPreferencesFrame.this, UIPreferencesFrame.GRS("AboutMessage"));  
				} 
			});
		}

	}

	public UIPreferencesFrame()  {
		super("UI Preferences");
		
		// Properties
		this.setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
		this.addWindowListener(new WindowAdapter() { public void windowClosing(WindowEvent e) { quit(); } });
		
		// add menus
		setJMenuBar(new MenuBar());
		
		// create and set panel
		setContentPane(new ContentPanel());
		
		// packen
		pack();
		prefs.add("PrefsFrame", this);
	}

	static String GRS(String id)  {
		try
			{
			return resources.getString(id);
			}
		catch (MissingResourceException x) 
			{
			System.err.println(x.getLocalizedMessage() + ": '" + id + "'" );
			return "MISSING RESOURCE: " + id;
			}
	}

	static String GRS(String id, String arg1)  {
		String res = GRS(id);
		
		try
			{
			res = MessageFormat.format(res, new Object [] { arg1 });
			}
		catch (Exception x) { System.err.println(x); }
		
		return res;
	}

	public static void main(String [] args)  {
		UIPreferences.init();
		
		final UIPreferencesFrame prefsFrame = new UIPreferencesFrame();
		prefsFrame.setVisible(true);
	}

	void quit()  {
		prefs.save();
		System.exit(0);
	}

}
